// AI_SoftwareTrigger_Log.cpp :
//
// HSDAQ data logger folder structure
//
//    +----\log
//          +----\xx.xx.xx.xx 
//                 +----\yyyy_mm_dd_hh_mm []
//                        +----xx_xx_xx_xx_4c42_yyyy_mm_dd_hh_mm_ss.bin [file name] 
//   
//    Note:
//       1.xx.xx.xx.xx : [The folder name is IP address of the PET-7H24M]
//       2.yyyy_mm_dd_hh_mm [It's the default folder format yyyy:year, mm:month, dd:day, hh:hour, mm:minute]
//       3.10_1_107_125_4c42_2018_11_30_14_16_29.bin [file name]  
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include "windows2linux.h"
#include <termios.h>//#include <conio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/select.h>

#include "hsdaql.h"

#define GetCurrentDirectory(a,b) getcwd(b,a)
I32 ReplaceChar(char *srcStr,char r, char t)
{
	I32 len=strlen(srcStr);
	I32 i;
	if(len)
	{
		for(i=0;i<len;i++)
		{
			if(srcStr[i]==r)
				srcStr[i]=t;
		}	
	}
	return 0;
}
void changemode(I32 dir)
{
  static struct termios oldt, newt;
 
  if ( dir == 1 )
  {
    tcgetattr( STDIN_FILENO, &oldt);
    newt = oldt;
    newt.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newt);
  }
  else
    tcsetattr( STDIN_FILENO, TCSANOW, &oldt);
}
 
I32 kbhit (void)
{
  struct timeval tv;
  fd_set rdfs;
 
  tv.tv_sec = 0;
  tv.tv_usec = 0;
 
  FD_ZERO(&rdfs);
  FD_SET (STDIN_FILENO, &rdfs);
 
  select(STDIN_FILENO+1, &rdfs, NULL, NULL, &tv);
  return FD_ISSET(STDIN_FILENO, &rdfs);
 
}            
 
UL32 GetTickCount();
I32 msleep(L32 msec);
#define Sleep msleep

I32 main(I32 argc, char *argv[])
{
	HANDLE hHS;
	//char IPadd[64]="\0";
    char *IPadd=NULL;
	char sdk_version[16]={0};
	char fw_version[32]={0};

	bool ret= false;
	I16 chCnt=8;
	I16  useGain=0;


	I16 extriggMode=0;
	bool continuousMode=false;
	L32 continuousTimeout=0;
	UL32 targetCnt=0;
	L32 sampleRate=0;

	//UL32 totalRead=0,totalReceived=0;
	I16 DataTransMethod=0;
	I16 AutoRun=0;
    float fAI[8];

	if(argc>1)
	{	
        printf("%d,%s\r\n",argc,argv[1]);
		//char   *pFilePathName; 
   
		//I32   nLen   =   wcslen(argv[1])+1;    
		//WideCharToMultiByte(CP_ACP,   0,   argv[1],   -1,   IPadd,  64,   NULL,   NULL);
		//printf("program start %s\r\n",IPadd);
        IPadd=argv[1];
	}
	else
	{
		printf("error argument\r\n");
		return 0;
	}

	char tmp[128]={0};
	sprintf(tmp,"%s,9999,10010",IPadd);
	//Step 1: Create a TCP connection with ET-7H24

    printf("ET-7H24 Analog Input Scan Testing [N sample mode & Software trigger]\r\n");
	HS_GetSDKVersion(sdk_version);
	printf("HSDAQ SDK Ver=%s\r\n",sdk_version);
    changemode(1); 
    
	hHS = HS_Device_Create(tmp);

	Sleep(1);
	if(hHS!=NULL)
	{
		HS_GetFirmwareVersion(hHS,fw_version);
		printf("ET7H24 Firmware Ver=%s\r\n",fw_version);

		chCnt=4;
		extriggMode=0;
		targetCnt=1000;
		sampleRate=10000; 
		ret=HS_SetAIScanParam(hHS, chCnt, useGain, extriggMode, sampleRate, targetCnt, DataTransMethod,AutoRun);
		//Step 2: SetScanParam sets chCnt, useGain, triggerMode, sampleRate, targetCnt to ET7H24
            /* For details, please refer to the API manual
             * AI CH input Range : 1 ~ 4
             * Gain Range        : 0 ~ 9
             * Mode Range        : 0 ~ 1
             * Sample rate       : 20 ~ 60K (continue mode)
             * targetcnt         : 0 (continue mode)
             * filetype          : 0B1B3B4
             * fileinterval      : min
            */      

		//Step 2: SetScanParam sets chCnt, useGain, triggerMode, sampleRate, targetCnt to ET7H24
		if(ret==false)
		{
			//printf("Error code 0x%x\r\n",HS_GetLastError());
            printf("Set AIscan param failed\r\n");
		}
		HS_GetAIScanParam(hHS, &chCnt, &useGain, &extriggMode, &sampleRate, &targetCnt,&DataTransMethod,&AutoRun); 
		//Step 3: GetScanParam getsx chCnt, useGain, triggerMode, sampleRate, targetCnt to check with ET7H24
		printf("ET-7H24 first Scan parameters chCnt %d, useGain %d, triggMode %d, sampleRate %ld, targetCnt %lu \n", chCnt, useGain, extriggMode, sampleRate, targetCnt);

		Sleep(10);
		//TCHAR tcfullPath[MAX_PATH];
		char  cfullPath[MAX_PATH];
		GetCurrentDirectory( MAX_PATH, cfullPath );
		//WideToChar(tcfullPath,cfullPath);
        printf(">>>%s\n",cfullPath);
	//Step 4: Start AI data logger
		ret=HS_StartLogger(hHS,cfullPath,2,0); // 2 mins
		//second paramter ==> NULL, it means the log file will saved to the current directory
		if(ret==false)
		{
			printf("Error code 0x%x\r\n",HS_GetLastError());
		}

		WORD BufferStatus=0;
		#define BUFFERSIZE 1000
		float fdataBuffer[BUFFERSIZE];
		UL32 ulleng=0;
		UL32 totalRecv=0;
		U32 remChannel=0;

		UL32 CH1checkerror=0;
		UL32 CH7checkerror=0;
		UL32 otherCHcheckerror=0;
		bool quit=false;
		U32 triggerStatus;

		//Step 5: while loop to get total received data count from ET-7H24
		while(quit==false)
		{				
			ret=HS_GetTotalSamplingStatus(hHS,&ulleng,&triggerStatus);
			if(ret==false)
			{
				printf("Error code 0x%x\r\n",HS_GetLastError());
			}
			else
			{                               
				if(ulleng>=targetCnt && targetCnt>0) //N sample mode
				{					
					//Step 6:When the total number of samples reaches the target counts. Send a command to get the all acquisition data over the Ethernet.                    
					ret=HS_TransmitDataCmd(hHS);
                    printf("Send command to get all acquisition data...\n");
					Sleep(6000);
					break;
				}
				else
					printf(".");
			} 
			if(kbhit())
			{
					//Sleep(1);
                char ch=getchar();
                printf("%c",ch);
			     //if(getchar()=='q' || getchar()=='Q')
                if(ch=='q' || ch=='Q')
                    goto END;
				//break;
			}						
			else
				Sleep(100);
		}
		//Step7: Stop AI data logger
		HS_StopLogger(hHS);
		printf("AP total read %lu\n", ulleng);
		//Step8: Close the connection
		HS_Device_Release(hHS);

#if 1
		ReplaceChar(IPadd,'.','_');
		strcat(cfullPath,"/log/");
		strcat(cfullPath,IPadd);
		
		I16 gchCnt=0;
		I16 guseGain=0;
		I16 gextriggMode=0;
		bool gcontinuousMode=false;
		L32 gcontinuousTimeout=0;
		UL32 gtargetCnt=0;
		L32 gsampleRate=0;
		I16 gDataTransMethod;
        I16 gSyncMode;
		U16 gVarr[4];
		I16 oVarr[4];        
		
		//Step 9: Get the total number of files in the specified folder according to the file type
		I32 ind=HS_GetAllLogFiles(cfullPath,0);
		printf("ind=%d\r\n",ind);
		if(ind>0)
		{
			for(I32 i=0;i<ind;i++)
			{
				//TCHAR tcgetfulfilelPath[MAX_PATH]={0};
                char tcgetfulfilelPath[MAX_PATH]={0}; 
				
				//Step 10: Open log file by the index number that indicates the order in which the file is located in the specified folder
				HANDLE hlf=HS_LogFile_Open_byIndex(i,tcgetfulfilelPath);
				printf("%d ==>%s\r\n",i,tcgetfulfilelPath);
				//Step 11:Get chCnt, useGain, triggerMode, sampleRate, targetCnt parameters of the selected log file
				HS_GetLogFile_AIScanConfigInfo(hlf,&gchCnt,&guseGain,&gextriggMode,&gsampleRate,&gDataTransMethod,&gSyncMode);
                                
				printf("%d, %d, %d, %ld, %d, %d\r\n",gchCnt,guseGain,gextriggMode,gsampleRate,gDataTransMethod,gSyncMode);
				
				U16 gainVal=0;
				I16 offsetVal=0;
				//Step 12:Get gain/offset parameters of the selected log file
				for(I32 i=0;i<gchCnt;i++)
				{
				    HS_GetLogFile_GainOffset(hlf,i,guseGain, &gVarr[i], &oVarr[i]);
					printf("%d, %d, %d\r\n",i,gVarr[i],oVarr[i]);
				}
				DWORD filesize;
				I32 filetype;
				I32 fileversion;
                char logfilename[MAX_PATH]; 
				//Step 13:Get the file information of the selected log file
				HS_GetLogFileInfo(hlf,logfilename,&filesize,&filetype,&fileversion);
                
				printf("%ld, %d\r\n",filesize,filetype);
				DWORD samplecount;
				char startdate[32],starttime[32];
				//Step 14:Get the total sampling counts and the date/time of the first data acquiring in the selected log file
				HS_GetLogFile_AIScanSampleInfo(hlf,&samplecount,startdate,starttime);
				printf("%ld, %s %s\r\n",samplecount,startdate,starttime);
				L32 *databuff=(L32 *)malloc(sizeof(L32)*samplecount);
				//Step 15:Get the total data of the selected log file
				HS_GetLogFile_AIDataHex(hlf,0,samplecount,databuff);
                for(I32 aa=0;aa<samplecount;aa++)
				{
					
					I32 sch = aa % gchCnt;
					float fdata;
					HS_Calibrate_Data_Float(hlf,sch,guseGain,databuff[aa],&fdata);
					if (sch == (gchCnt-1)) //new line
                    {
						printf("%f\r\n",fdata);
                    }
                    else
                    {
						printf("%f,",fdata);    
                    }
					
				}
                
                free(databuff);
				//Step 16: Close the selected log file
				HS_LogFile_Close(hlf);
			}
		}                                     
#endif              
	}
	//Step7: Close the connection
END:        
	HS_Device_Release(hHS);
         
    changemode(0);    
	return 0;
}     

